function readlpp(fname)
% READLPP    Reading and converting an LPP-file into default.mat.

% Detong Zhang & Yin Zhang, July 1995, Dept Math/Stat, UMBC

global NAME variables vlp n_cons n_bnds n_vars
NAME=[]; vlp=[]; variables= ''; tmpvlp = [];
n_cons=0; n_bnds=0; n_vars=0;
DEFAULT = '/tmp/default';

if (nargin == 0) 
   fprintf('Default directory: lipsol/lppdir\n');
   fname = input('Problem name: ','s'); 
   fname = ['../lppdir/' fname];
end

check_num = 0; o_cont = 0; n_cons = 0; n_bnds = 0; cont = 0;
iscomment = 1; comment = [];

if isempty(findstr('.lpp',fname)) fname = [fname '.lpp']; end
fid = fopen(fname, 'r');
if fid  == -1 error(['Cannot open file ' fname]); end
while 1
  cont = cont + 1;
  string = fgetl(fid);
  if ~isstr(string) n_cons = n_cons-1; n_bnds = n_bnds-1;break; end
  if (isempty(string)) string = '%'; end
  if (string(1) == '%' & iscomment) 
     comment = str2mat(comment,string);
  end
  if (string(1) ~= '%') 
    len = length(string);

%   ...... error checking ......

    nb = find(string ~= ' ' & string ~= 0);
    n1 = nb(1);
    if ~( (string(n1) >= '0' & string(n1) <= '9') ...
         | strcmp(lower(string(n1:min(len,n1))),'m') ...
         | strcmp(lower(string(n1:min(len,n1+3))),'name') ...
         | strcmp(lower(string(n1:min(len,n1+2))),'obj') ...
         | strcmp(lower(string(n1:min(len,n1+2))),'con') ...
         | strcmp(lower(string(n1:min(len,n1+2))),'bou') )
      errmessage(['Incorrect format on line: ' num2str(cont)]);
      return;
    end

%   ...... checking and converting ......

    if strcmp(lower(string(n1:min(len,n1+3))),'name')    iscomment = 0;
    elseif strcmp(lower(string(n1:min(len,n1+2))),'obj') check_num = 1;
    elseif strcmp(lower(string(n1:min(len,n1+2))),'con') check_num = 2;
    elseif strcmp(lower(string(n1:min(len,n1+2))),'bou') check_num = 3;

%   ...... constraints converting when meet 'bounds:' ......

      n_vars = 0;
      tmpvlp(1,:) = [];
      A = [];
      b = [];
      c = [];
      [NAME,c,variables,n_vars,BIG]= transobj(tmpvlp,c,variables,n_vars);
      [A,b,variables,n_vars] = ...
               transcon(A,b,tmpvlp,1:n_cons-1,n_cons-1,variables,n_vars);
      c = [c; zeros(n_vars-length(c),1)];
      lv = zeros(n_vars,1);
      rv = lv;
    else

%     ...... error checking ......

      if (check_num > 1)
         ss = find(string == ')');
         string = string((ss(1)+1:len)) ;
      end
      if (errcheck(string,check_num)) 
        errmessage(['Illegal input in ' fname ', line: ' num2str(cont)]);
        return;
      end
      if (check_num == 3)
        [err,lv,rv] = checkbnds([') ' string],lv,rv);
        if (err)
          errmessage(['Bound error in ' fname ', line: ' num2str(cont)]);
          return;
        end
      end
    end % if-else

% ...... put the string into tmpvlp after the string checking ......

    string = string(find(string ~= ' ' & string ~= 0));
    if (check_num == 0)
      string = strrep(string,':',': ');
    elseif (check_num == 1)
      len = length(string);
      if (o_cont ~= 0) string = ['  ' string(1:3) ' ' string(4:len)]; end
      o_cont = o_cont + 1;
    elseif (check_num == 2)
      if (n_cons ~= 0) string = ['  ' num2str(n_cons) ') ' string]; end
      n_cons = n_cons + 1;
    elseif (check_num == 3)
      if (n_bnds ~= 0) string = ['  ' num2str(n_bnds) ') ' string]; end
      n_bnds = n_bnds +1;
    end
       string = strrep(string,'+',' + ');
       string = strrep(string,'-',' - ');
       string = strrep(string,'=',' = ');
       string = strrep(string,'<',' < ');
       string = strrep(string,'>',' > ');
       string = strrep(string,'}','} ');
       tmpvlp = str2mat(tmpvlp, string);
   end % (if ~=%)
end % end while
fclose(fid);

% ...... bounds converting and save whole input ......

lbounds = zeros(n_vars,1);
ubounds = BIG*ones(n_vars,1);
[lbounds,ubounds,variables,n_vars] = ...
  transbnd(lbounds,ubounds,tmpvlp,1:n_bnds,n_cons,variables,n_vars,BIG);
vlp = str2mat(comment, tmpvlp); vlp(1,:) = [];
eval(['save ' DEFAULT ' A b c lbounds ubounds BIG NAME']);
